    /**************************************************************************
 *                                                                         *
 * SAS_1.cpp				                                               *
 *                                                                         *
 ***************************************************************************
 *                                                                         * 
 * Created: 1/9/2013			                                           *
 *                                                                         *
 * Mission program for LTRL Project Maverick Stand Along SMD			   *
 ***************************************************************************/


#define F_CPU 16000000
#define ARDUINO 100

#include "Arduino.h"
#include "Wire.h"
#include "SD.h"
#include "RTCLib.h"
#include "BMPLib.h"
#include "ADXL345.h"
#include "HIHLib.h"
#include "avr/sleep.h"

File logFile;
File gpsFile;
RTCLib rtc;
BMPLib bmp;
ADXL345 adxl;			
HIHLib hih;

	//////////////////////////////////////////
	//////////////// VARIABLES ///////////////
	//////////////////////////////////////////

#define testSwitch 3	// Program determination variables
#define standBySwitch 4
#define TESTLED 5			// Tri-state LED pins
#define RUNLED 6
#define STANDBYLED 7

int mission = 0;
int missionStage = 0;
#define STANDBYMODE 0
#define TESTMODE 1
#define RUNMODE 2
#define ONSTAND 0
#define POWASC 1
#define APPOGEE 2
#define LANDING 3
#define TIMECOM 4
#define MISSCOM 5

boolean val;
boolean error_found = false;
	//////////////// Pressure/Temperature Sensor ///////////////
float temprature;
float pressure;
float altitude_meters;
float altitude_feet;
	///////////// Accelerometer ////////////
int x_acc;
int y_acc;
int z_acc;
	/////////// Humidity Sensor //////////

	/////////// Light Sensor //////////
int lightSensorValue;
	//////////////// GPS ///////////////
char gps_line[300];
char comandoGPR[7] = "$GPRMC";
	//////////////// SD ///////////////

	/////////////// RTC //////////////
	
	/////////////// MISSION //////////////
int high_altitude = 0;
int past_apogee_counter = 0;
float last_read = 0;
float landing_timer = 0;
	

	//////////////////////////////////////////
	//////////////// FUNCTIONS ///////////////
	//////////////////////////////////////////
void LED_init(void);
void component_init(void);
void set_indicator(int);
void LED_Flash(int);
void read_data(void);
void data_sd(void);
void read_gps(void);
void gpd_sd(void);
void print_data(void);
void read_lightsensor(void);
void run_mode(void);
void setup();
void loop();

void setup() {
	Serial.begin(9600);
	Serial.println("Starting Setup");
	LED_init();
	component_init();

	set_sleep_mode(SLEEP_MODE_PWR_DOWN);			// Prepare Sleep Mode
	Serial.println("Setup Complete");
}

void loop(){
	
	if(digitalRead(standBySwitch)){
		mission = STANDBYMODE;
	}else{
		if(digitalRead(testSwitch)){
			mission = TESTMODE;
		}else{
			mission = RUNMODE;
		}
	}		
	

	switch(mission){
		case STANDBYMODE:
				set_indicator(STANDBYMODE);
			break;
			
		case TESTMODE:
				set_indicator(TESTMODE);
					read_data();
					print_data();
					data_sd();
                                        read_gps();
                                        gps_sd();
			break;
		
		case RUNMODE:
				set_indicator(RUNMODE);
				run_mode();
			break;
		default: break;
	}

}




	/////////////////////////////////////////////
	//////////////EXTERNAL FUNCTIONS/////////////
	/////////////////////////////////////////////
void LED_init(void){
	pinMode(testSwitch, INPUT);			// Three switches
	pinMode(standBySwitch, INPUT);
	pinMode(STANDBYLED, OUTPUT);			// Tri-state LED - LOW = ON, HIGH = OFF
	pinMode(TESTLED, OUTPUT);
	pinMode(RUNLED, OUTPUT);
	digitalWrite(STANDBYLED, HIGH);
	digitalWrite(TESTLED, HIGH);
	digitalWrite(RUNLED, HIGH);
}		

void component_init(void){
	Serial.println("Starting Component Initialization");
                /*
                Serial.print("Initializing RTC: ");								// RTC
		if(rtc.begin()){
			Serial.println("Successful");
		}else{
                    error_found = true;
                }
		Serial.print("To program the RTC enter \"program\" now: ");
		delay(5000);
		if (Serial.read() == 'program') {
			Serial.println("Programming");
			rtc.program_RTC();
		}else{
			Serial.println("Programming Skipped");
		}
		*/
		Serial.print("Initializing BMP085: ");							// BMP085
		if(bmp.begin()){
			bmp.calibrate();
			Serial.println("Successful");
		}else{
                    error_found = true;
                }
                
		
		Serial.print("Initializing ADXL345: ");							// ADXL345
		adxl.powerOn();
		if(adxl.adxlinit()){
			Serial.println("Successful");		// Sets all initializations and limit values - see ADXL345.cpp to adjust values
		}else{
                    error_found = true;
                }
                		

		Serial.print("Initializing HIH6131: ");							// HIH6131
		if(hih.begin()){
			Serial.println("Successful");
		}else{
                    error_found = true;
                }

                Serial.print("Initializing SD card: "); 
                 pinMode(8, OUTPUT);
   
                 if (SD.begin(8)) {
                   Serial.println("Successful");
                 }else{
                    error_found = true; goto ERROR;
                }
ERROR:
                while(error_found){
                    digitalWrite(STANDBYLED, HIGH);		// set tri-state leds
		    digitalWrite(TESTLED, LOW);
		    digitalWrite(RUNLED, HIGH);
                }
       
          Serial.println("Component Initialization Complete");
}	

void set_indicator(int mode){
	switch(mode){
		case STANDBYMODE:
		  digitalWrite(STANDBYLED, HIGH);		// set tri-state leds
		  digitalWrite(TESTLED, HIGH);
		  digitalWrite(RUNLED, HIGH);
                  LED_Flash(STANDBYLED);
		  break;
		case TESTMODE:
		  digitalWrite(STANDBYLED, HIGH);		// set tri-state leds
		  digitalWrite(TESTLED, HIGH);
		  digitalWrite(RUNLED, HIGH);
                  LED_Flash(TESTLED);
		  break;
		case RUNMODE:
		  digitalWrite(STANDBYLED, HIGH);		// set tri-state leds
		  digitalWrite(TESTLED, HIGH);
		  digitalWrite(RUNLED, LOW);
		  break;
		default: break;
	}
}

void LED_Flash(int LED){
             digitalWrite(LED, LOW);
             delay(100);
             digitalWrite(LED, HIGH);
             delay(100);
             digitalWrite(LED, LOW);
             delay(100);
             digitalWrite(LED, HIGH);
             delay(700);
}

void read_lightsensor(void){
	lightSensorValue = 0;
	lightSensorValue = analogRead(A3);    
}
	
void read_data(void){
		//rtc.RTC_printDate();					// RTC - Saves variables and prints to serial
                bmp.read_values();					// BMP085 - Reads data and stores in variable altitude_feet (or altitude_meters)
                adxl.readAccel(&x_acc, &y_acc, &z_acc);	                // ADXL - Saves values  in given variables
                hih.read_data();					// HIH6131 - Reads data and stores in variables RH and T_C
                read_lightsensor();					// TEMT6000 - Reads data and stores in variable lightSensorValue
		//read_gps();
}


void print_data(void){			
												// Function read_data must be called first!
	// print the date and time 23:59:59 3/1/11 
	Serial.print(rtc.hour);
	Serial.print(":");
	Serial.print(rtc.minute);
	Serial.print(":");
	Serial.print(rtc.second);
	Serial.print(" ");
	Serial.print(rtc.month);
	Serial.print("/");
	Serial.print(rtc.monthDay);
	Serial.print("/");
	Serial.print(rtc.year);
	// print BMP085 data
	//Serial.print(" Pressure: ");
	//Serial.print(bmp.pressure);
	Serial.print(" Temperature: ");
	Serial.print(bmp.temperature);
	Serial.print(" Altitude(m): ");
	Serial.print(bmp.altitude_meters);
	// print ADXL345 data
	Serial.print(" Acc_X: ");
	Serial.print(x_acc);
	Serial.print(", Acc_Y: ");
	Serial.print(y_acc);
	Serial.print(", Acc_Z: ");
	Serial.print(z_acc);
	// print HIH6131 data
	Serial.print(" Humidity: ");
	Serial.print(hih.RH);
	//Serial.print(" Temperature: ");
	//Serial.print(hih.T_C);
	// print Light Sensor value
	Serial.print(" Lightlvl: ");
	Serial.print(lightSensorValue);
	Serial.print(" Stage: ");
	Serial.println(missionStage);
}

void data_sd(void){
     logFile = SD.open("log.txt", FILE_WRITE);
     
     if(logFile){
        logFile.print(rtc.hour);
	logFile.print(":");
	logFile.print(rtc.minute);
	logFile.print(":");
	logFile.print(rtc.second);
	logFile.print(" ");
	logFile.print(rtc.month);
	logFile.print("/");
	logFile.print(rtc.monthDay);
	logFile.print("/");
	logFile.print(rtc.year);
	// print BMP085 data
	logFile.print(" Pressure: ");
	logFile.print(bmp.pressure);
	logFile.print(" Temperature: ");
	logFile.print(bmp.temperature);
	logFile.print(" Altitude(feet): ");
	logFile.print(bmp.altitude_feet);
	// print ADXL345 data
	logFile.print(" Acc_X: ");
	logFile.print(x_acc);
	logFile.print(", Acc_Y: ");
	logFile.print(y_acc);
	logFile.print(", Acc_Z: ");
	logFile.print(z_acc);
	// print HIH6131 data
	logFile.print(" Humidity: ");
	logFile.print(hih.RH);
	//logFile.print(" Temperature: ");
	//logFile.print(hih.T_C);
	// print Light Sensor value
	logFile.print(" Lightlvl: ");
	logFile.print(lightSensorValue);
	logFile.print(" Stage: ");
	logFile.println(missionStage);
        logFile.close();
     }else{
       Serial.println("Error opening log.txt");
     }
}


void read_gps(void){

	// Setup - Don't know if i need this, saving it later for after debugging
	//for (int i=0;i<300;i++){       // Initialize a buffer for received data
	//	gps_line[i]=' ';
	//}

	// Read							// Reads serial_in until $ is found, then reads until line ends and stores in gps_line
	Serial.println("Starting read_GPS");								// may want to add so it only reads specific lines (if GPS isn't adjusted itself)
	if(Serial.available() > 0){
            Serial.println("if Serial finds &)");
		if(Serial.find("$")){
                     Serial.println("Reading gps_line");
			Serial.readBytesUntil('\n', gps_line, 300);
		}
	}	
        Serial.println("Done read_GPS");
	
}

void gps_sd(void){
     gpsFile = SD.open("gps.txt", FILE_WRITE);
     
     if(gpsFile){
        gpsFile.print(gps_line);
        gpsFile.close();
     }else{
       Serial.println("Error opening gps.txt");
     }
}

void run_mode(void){
	
	switch(missionStage){
		
		case ONSTAND:										// Read ADXL345 values - Check ADXL345 for powered ascent - Increment mission stage
		     digitalWrite(STANDBYLED, HIGH);
		     digitalWrite(TESTLED, HIGH);
                     LED_Flash(RUNLED);
                        //adxl.readAccel(&x_acc, &y_acc, &z_acc);			
			if(x_acc == 3 || y_acc == 3 || z_acc == 3){		
				Serial.print("Powered ascent detected: ");
				Serial.print("Acc_X: ");
				Serial.print(x_acc);
				Serial.print(", Acc_Y: ");
				Serial.print(y_acc);
				Serial.print(", Acc_Z: ");
				Serial.println(z_acc);
				missionStage++;								
			}
		break;
		
		case POWASC:										// Read BMP085 values - Check for apogee - Increment mission stage
			set_indicator(RUNMODE);
                        bmp.read_values();
			if(bmp.altitude_feet > high_altitude){
				high_altitude = bmp.altitude_feet;
			}else{
				past_apogee_counter++;
			}
			
			if(past_apogee_counter > 20){
				missionStage++;
			}								
		
			
		break;
		
		case APPOGEE:										// If last reading was more than 5 seconds ago - Read data and store
			if((millis() - last_read) > 5000){				
				last_read = millis();						
				read_data();
				data_sd();
				print_data();
			}
				
			bmp.read_values();								// Read BMP085 values - if last altitude is within +/- 5 feet of current altitude - increment mission stage
			if((bmp.last_altitude_feet - bmp.altitude_feet) < 5 && (bmp.last_altitude_feet - bmp.altitude_feet) > -5){	
				missionStage++;																						
				landing_timer = millis() + 600000;			// Set pointer for 10 minutes after landing
			}
		break;
		
		case LANDING:										// If last reading was more than 60 seconds ago - Read data and store
			if((millis() - last_read) > 60000){			
				last_read = millis();						
				read_data();
				data_sd();
				print_data();
			}
															// If timer pointer has been reached - increment mission stage
			if(millis() > landing_timer){					
				missionStage++;								
			}
		break;
		
		case TIMECOM:
			logFile = SD.open("log.txt");
                        if(logFile){
                         while(logFile.available()){
                           Serial.write(logFile.read());
                         } 
                         logFile.close();
                        }else{
                          Serial.println("Error returning log.txt");
                        }
                        
		break;
		
		case MISSCOM:
			sleep_enable(); 
			sleep_mode();
		break;
		
		default: break;
		
	}
	
}




