    /***********************************************************************
 *                                                                         *
 * SAS_1.cpp				                                   *
 *                                                                         *
 ***************************************************************************
 *                                                                         * 
 * Created: 1/9/2013			                                   *
 *                                                                         *
 * Mission program for LTRL Project Maverick Stand Along SMD	           *
 ***************************************************************************/

#define F_CPU 16000000
#define ARDUINO 100

#include "Arduino.h"
#include "Wire.h"
#include "SD.h"
#include "RTClib.h"
#include "BMPLib.h"
#include "ADXL345.h"
#include "HIHLib.h"
#include "avr/sleep.h"

RTC_DS1307 rtc;
BMPLib bmp;
ADXL345 adxl;			
HIHLib hih;

	//////////////////////////////////////////
	//////////////// VARIABLES ///////////////
	//////////////////////////////////////////

#define testSwitch 3	// Program determination variables
#define standBySwitch 4
#define TESTLED 5	// Tri-state LED pins
#define RUNLED 6
#define STANDBYLED 7

int mission = 0;
int missionStage = 0;
#define STANDBYMODE 0
#define TESTMODE 1
#define RUNMODE 2
#define ONSTAND 0
#define POWASC 1
#define APPOGEE 2
#define LANDING 3
#define TIMECOM 4
#define MISSCOM 5

boolean val;
boolean error_found = false;
	//////////////// Pressure/Temperature Sensor ///////////////
float temperature;
float pressure;
float altitude_meters;
float last_altitude_meters;
float real_altitude_meters;
float altitude_feet;
	///////////// Accelerometer ////////////
int acc[3];
int G[3];
double gains[3] = {0.0078, 0.0078, 0.0078};
	/////////// Humidity Sensor //////////

	/////////// Light Sensor //////////
int lightSensorValue;
	//////////////// GPS ///////////////
#define BUFF_MAX 100    // size of GPS & SD buffers
char inBuffer[BUFF_MAX];    // buffer used to read NMEA lines from GPS
byte outBuffer[BUFF_MAX];   // buffer used to write NMEA lines to SD card
char gps_line[300];
//char comandoGPR[7] = "$GPRMC";
	//////////////// SD ///////////////
File logFile;
File gpsFile;
	/////////////// RTC //////////////
DateTime now;
	/////////////// MISSION //////////////
int high_altitude = 0;
int past_apogee_counter = 0;
float last_read = 0;
float landing_timer = 0;
	

	//////////////////////////////////////////
	//////////////// FUNCTIONS ///////////////
	//////////////////////////////////////////
void LED_init(void);
void component_init(void);
void set_indicator(int);
void LED_Flash(int);
void read_data(void);
void data_sd(void);
void read_gps(void);
void gps_sd(void);
void print_data(void);
void read_lightsensor(void);
void run_mode(void);
void setup();
void loop();

void setup() {
	Serial.begin(57600);
	Serial.println("Starting Setup");
	LED_init();
	component_init();
        Serial.setTimeout(1000);
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);			// Prepare Sleep Mode
	Serial.println("Setup Complete");
}

void loop(){
	
	if(digitalRead(standBySwitch)){
		mission = STANDBYMODE;
	}else{
		if(digitalRead(testSwitch)){
			mission = TESTMODE;
		}else{
			mission = RUNMODE;
		}
	}		
	

	switch(mission){
		case STANDBYMODE:
				set_indicator(STANDBYMODE);
			break;
			
		case TESTMODE:
				set_indicator(TESTMODE);
					read_data();
					print_data();
					data_sd();
                                        read_gps();
                                        gps_sd();
			break;
		
		case RUNMODE:
				set_indicator(RUNMODE);
				run_mode();
			break;
		default: break;
	}

}




	/////////////////////////////////////////////
	//////////////EXTERNAL FUNCTIONS/////////////
	/////////////////////////////////////////////
void LED_init(void){
	pinMode(testSwitch, INPUT);			// Three switches
	pinMode(standBySwitch, INPUT);
	pinMode(STANDBYLED, OUTPUT);			// Tri-state LED - LOW = ON, HIGH = OFF
	pinMode(TESTLED, OUTPUT);
	pinMode(RUNLED, OUTPUT);
	digitalWrite(STANDBYLED, HIGH);
	digitalWrite(TESTLED, HIGH);
	digitalWrite(RUNLED, HIGH);
}		

void component_init(void){
	Serial.println("Starting Component Initialization"); 
                  digitalWrite(STANDBYLED, LOW);  digitalWrite(TESTLED, HIGH);  digitalWrite(RUNLED, HIGH);		// set tri-state leds
	          
                Serial.print("Initializing SD card: ");         // SD Card
                 pinMode(8, OUTPUT);
   
                 if (SD.begin(8)) {
                   Serial.println("Successful");
                 }else{
                    error_found = true; goto ERROR;
                }
                logFile = SD.open("log.txt", FILE_WRITE);
                
                Serial.print("Initializing DS1307: ");      // RTC	
                logFile.print("Initializing DS1307: ");							
		if(rtc.begin()){
			Serial.println("Successful");
                        logFile.println("Successful");
		}else{
                    error_found = true;
                }
                
		Serial.print("Initializing BMP085: ");							// BMP085
		logFile.print("Initializing BMP085: ");
                if(bmp.begin()){
			//bmp.calibrate();
			Serial.println("Successful");
                        logFile.println("Successful");
		}else{
                    error_found = true;
                }
                
		
		Serial.print("Initializing ADXL345: ");							// ADXL345
		logFile.print("Initializing ADXL345: ");	
		adxl.powerOn();
		if(adxl.adxlinit()){
                        adxl.setAxisGains(&gains[0]);
			Serial.println("Successful");		// Sets all initializations and limit values - see ADXL345.cpp to adjust values
			logFile.println("Successful");
		}else{
                    error_found = true;
                }
                		

		Serial.print("Initializing HIH6131: ");							// HIH6131
		logFile.print("Initializing HIH6131: ");
		if(hih.begin()){
			Serial.println("Successful");
			logFile.println("Successful");
		}else{
                    error_found = true;
                }


ERROR:
                while(error_found){
                    digitalWrite(STANDBYLED, HIGH);		// set tri-state leds
		    digitalWrite(TESTLED, LOW);
		    digitalWrite(RUNLED, HIGH);
                }
       
          Serial.println("Component Initialization Complete");
          logFile.println("Component Initialization Complete");
          logFile.close();
}	

void set_indicator(int mode){
	switch(mode){
		case STANDBYMODE:
		  digitalWrite(STANDBYLED, HIGH);		// set tri-state leds
		  digitalWrite(TESTLED, HIGH);
		  digitalWrite(RUNLED, HIGH);
                  LED_Flash(STANDBYLED);
		  break;
		case TESTMODE:
		  digitalWrite(STANDBYLED, HIGH);		// set tri-state leds
		  digitalWrite(TESTLED, HIGH);
		  digitalWrite(RUNLED, HIGH);
                  LED_Flash(TESTLED);
		  break;
		case RUNMODE:
		  digitalWrite(STANDBYLED, HIGH);		// set tri-state leds
		  digitalWrite(TESTLED, HIGH);
		  digitalWrite(RUNLED, LOW);
		  break;
		default: break;
	}
}

void LED_Flash(int LED){
             digitalWrite(LED, LOW);
             delay(100);
             digitalWrite(LED, HIGH);
             delay(100);
             digitalWrite(LED, LOW);
             delay(100);
             digitalWrite(LED, HIGH);
             delay(700);
}

void read_lightsensor(void){
	lightSensorValue = 0;
	lightSensorValue = analogRead(A3);    
}
	
void read_data(void){
		now = rtc.now();				// RTC - Saves variables and prints to serial
                temperature = bmp.readTemperature();		        // BMP085 - Reads data and stores in variable altitude_feet (or altitude_meters)
                pressure = bmp.readPressure();
                altitude_meters = bmp.readAltitude();
                adxl.readAccel(&acc[0]);	                // ADXL - Saves values  in given variables
                hih.read_data();					// HIH6131 - Reads data and stores in variables RH and T_C
                read_lightsensor();					// TEMT6000 - Reads data and stores in variable lightSensorValue
		read_gps();
                gps_sd();
}


void print_data(void){			
												// Function read_data must be called first!
	// print the date and time 23:59:59 3/1/11 
	Serial.print(now.month(), DEC);
	Serial.print("/");
	Serial.print(now.day(), DEC);
	Serial.print("/");
	Serial.print(now.year(), DEC);
        Serial.print("\t");
	Serial.print(now.hour(), DEC);
	Serial.print(":");
	Serial.print(now.minute(), DEC);
	Serial.print(":");
	Serial.print(now.second(), DEC);
	Serial.print("\t");
	// print BMP085 data
	//Serial.print(" Pressure: ");
	//Serial.print(pressure);
	//Serial.print("\t");
	Serial.print("Temperature: ");
	Serial.print(temperature);
        Serial.print("\t");
	Serial.print("Altitude(m): ");
	Serial.print(altitude_meters);
        Serial.print("\t");
	// print ADXL345 data
	Serial.print("Acc_X: ");
	Serial.print(acc[0]);
        Serial.print("\t");
	Serial.print("Acc_Y: ");
	Serial.print(acc[1]);
        Serial.print("\t");
	Serial.print("Acc_Z: ");
	Serial.print(acc[2]);
        Serial.print("\t");
	// print HIH6131 data
	Serial.print("Humidity: ");
	Serial.print(hih.RH);
        Serial.print("\t");
	//Serial.print("Temperature: ");
	//Serial.print(hih.T_C);
        //Serial.print("\t");
	// print Light Sensor value
	Serial.print("Lightlvl: ");
	Serial.print(lightSensorValue);
        Serial.print("\t");
	Serial.print("Stage: ");
	Serial.println(missionStage);
}

void data_sd(void){
     logFile = SD.open("log.txt", FILE_WRITE);
     
     if(logFile){
        logFile.print(now.hour(), DEC);
	logFile.print(":");
	logFile.print(now.minute(), DEC);
	logFile.print(":");
	logFile.print(now.second(), DEC);
	logFile.print("\t");
	logFile.print(now.month(), DEC);
	logFile.print("/");
	logFile.print(now.day(), DEC);
	logFile.print("/");
	logFile.print(now.year(), DEC);
	logFile.print("\t");
	// print BMP085 data
	//logFile.print("Pressure: ");
	//logFile.print(pressure);
	//logFile.print("\t");
	logFile.print("Temperature: ");
	logFile.print(temperature);
	logFile.print("\t");
	logFile.print("Altitude(m): ");
	logFile.print(altitude_meters);
	logFile.print("\t");
	// print ADXL345 data
	logFile.print("Acc_X: ");
	logFile.print(acc[0]);
	logFile.print("\t");
	logFile.print("Acc_Y: ");
	logFile.print(acc[1]);
	logFile.print("\t");
	logFile.print("Acc_Z: ");
	logFile.print(acc[2]);
	logFile.print("\t");
	// print HIH6131 data
	logFile.print("Humidity: ");
	logFile.print(hih.RH);
	logFile.print("\t");
	//logFile.print("Temperature: ");
	//logFile.print(hih.T_C);
	//logFile.print("\t");
	// print Light Sensor value
	logFile.print("Lightlvl: ");
	logFile.print(lightSensorValue);
	logFile.print("\t");
	logFile.print("Stage: ");
	logFile.println(missionStage);
	logFile.print("\t");
        logFile.close();
     }else{
       Serial.println("Error opening log.txt");
     }
}


void read_gps(void){
/*
	// Setup - Don't know if i need this, saving it later for after debugging
	//for (int i=0;i<300;i++){       // Initialize a buffer for received data
	//	gps_line[i]=' ';
	//}
*/
	// Read							// Reads serial_in until $ is found, then reads until line ends and stores in gps_line
	if(Serial.available()){
		if(Serial.find("$")){
			Serial.readBytesUntil('\n', &gps_line[0], BUFF_MAX);    // If the first character of MNEA statement is found, read into buffer
		}
	}	

}

void gps_sd(void){

  gpsFile = SD.open("gps.txt", FILE_WRITE);
     
     if(gpsFile){
        for (int i = 0; i < BUFF_MAX; i++){    // Open gps.txt and store gps_line in it
          gpsFile.write(gps_line[i]);
          gps_line[i] = 0;                     // reset buffer to 0 to prevent false info
        }
          gpsFile.write('\n');
          gpsFile.close();
     }else{
       Serial.println("Error opening gps.txt");
     }

}

void run_mode(void){
	
	switch(missionStage){
		
		case ONSTAND:										// Read ADXL345 values - Check ADXL345 for powered ascent - Increment mission stage
		     digitalWrite(STANDBYLED, HIGH);
		     digitalWrite(TESTLED, HIGH);
                     LED_Flash(RUNLED);
                        adxl.readAccel(&acc[0]);			
			if(acc[2] > 280){	
                             logFile = SD.open("log.txt", FILE_WRITE);
				Serial.print("Powered ascent detected: ");
                                logFile.print("Powered ascent detected: ");
				Serial.print("Acc_Z: ");
                                logFile.print("Acc_Z: ");
				Serial.println(acc[2]);
                                logFile.println(acc[2]);
                              logFile.close();
		            missionStage++;								
			}
		break;
		
		case POWASC:										// Read BMP085 values - Check for apogee - Increment mission stage
			set_indicator(RUNMODE);
                        altitude_meters = bmp.readAltitude();
			  if(altitude_meters > high_altitude){
				high_altitude = altitude_meters;
			  }else{
				past_apogee_counter++;
			  }
			
			  if(past_apogee_counter > 20){
				missionStage++;
                                Serial.print("Apogee Reached, High altitude: ");
                                Serial.println(high_altitude);
			  }								
		
			
		break;
		
		case APPOGEE:										// If last reading was more than 5 seconds ago - Read data and store
			if((millis() - last_read) > 5000){				
				last_read = millis();						
				read_data();
				data_sd();
				print_data();
			}
												// Read BMP085 values - if last altitude is within +/- 5 feet of current altitude - increment mission stage
                        altitude_meters = bmp.readAltitude();
                        if((last_altitude_meters - altitude_meters) < 1 && (last_altitude_meters - altitude_meters) > -1){	
				missionStage++;
                                Serial.println("Vehicle Landed");										
				landing_timer = millis() + 600000;			// Set pointer for 10 minutes after landing
			}
                        last_altitude_meters = altitude_meters;
		break;
		
		case LANDING:										// If last reading was more than 60 seconds ago - Read data and store
			if((millis() - last_read) > 60000){			
				last_read = millis();						
				read_data();
				data_sd();
				print_data();
			}
															// If timer pointer has been reached - increment mission stage
			if(millis() > landing_timer){					
				missionStage++;								
			}
		break;
		
		case TIMECOM:
			logFile = SD.open("log.txt");
                            if(logFile){
                                while(logFile.available()){
                                 Serial.write(logFile.read());
                                 } 
                               logFile.close();
                            }else{
                              Serial.println("Error returning log.txt");
                            }
                       Serial.println("Timer has ended, system shutting down");
                       missionStage++;
		break;
		
		case MISSCOM:
			sleep_enable(); 
			sleep_mode();
		break;
		
		default: break;
		
	}
	
}




